define([
    'testsuite/qunit',
    'testsuite/qunit-fixture!de_epages/contextualappads/ui/appsdialog-test',
    'testsuite/sinon',
    'jquery',
    'de_epages/contextualappads/ui/appsdialog',

    '$ready!'
], function (QUnit, fixture, sinon, $, de_epages) {

    var mockApps = [{
            appUrl: 'https://www.billbee.de',
            description: 'Billbee bietet eine umfangreiche aber einfach zu bedienende...',
            id: '1',
            name: 'Billbee',
            installed: false,
            logoUrl: 'bild1.jpg'
        }, {
            appUrl: 'https://www.facebook.de',
            description: 'Billbee bietet eine umfangreiche aber einfach zu bedienende...',
            id: '2',
            name: 'Facebook',
            installed: false,
            logoUrl: 'bild2.jpg'
        }, {
            appUrl: 'https://www.whatsapp.de',
            description: 'Billbee bietet eine umfangreiche aber einfach zu bedienende...',
            id: '3',
            name: 'WhatsApp',
            installed: false,
            logoUrl: 'bild3.jpg'
        }]

    QUnit.module('de_epages.contextualappads.ui.appsdialog', {
        setup: fixture.init,
        teardown: fixture.destroy
    });

    QUnit.asyncTest("widget test presence of methods", function () {

        $.each([
            '_create',
            '_createDialog',
            '_initSlider',
            '_getPosition',
            'open',
            'close',
            'destroy'
        ], function (i, fn) {
                QUnit.equal(
                $.isFunction($.ui.contextualappadsUiAppsdialog.prototype[fn]),
                true,
                '$.isFunction(' + fn + '):'
            );
        });

        QUnit.start();
    });

    QUnit.asyncTest("widget ui tests", function () {

        var $div = $('.ep-top-bar-buttons-context-apps'),
            resizeSpy,
            appLinks,
            appImgs,
            i;

        $div.contextualappadsUiAppsdialog({
            apps: mockApps,
            shopId: 15
        });

        setTimeout(function() {
            QUnit.ok(
                $div.data('uiContextualappadsUiAppsdialog') !== undefined,
                'Widget exists'
            );

            // register spy on _getPosition method => number of executions can be investigated
            resizeSpy = sinon.spy($div.data('uiContextualappadsUiAppsdialog'), '_getPosition');

            QUnit.ok(
                $div.data('uiContextualappadsUiAppsdialog').dialog === undefined,
                'Dialog instance does not exist'
            );

            // Trigger click-event to open dialog
            $div.trigger('click');

            setTimeout(function() {
                QUnit.ok(
                    $div.data('uiContextualappadsUiAppsdialog').dialog !== undefined,
                    'Dialog instance exists'
                );
                QUnit.ok(
                    $div.data('uiContextualappadsUiAppsdialog').dialog.is(':visible') ,
                    'Dialog is visible'
                );
                QUnit.equal(
                    $div.data('uiContextualappadsUiAppsdialog').dialog.find('.ep-appdialog-slider-wrapper .ep-appdialog-slider-teaser').length,
                    mockApps.length,
                    'Dialog contains ' + mockApps.length + ' Apps'
                );

                QUnit.ok(
                    resizeSpy.calledTwice,
                    'Dialogs position was calculated.'
                );

                appLinks = $div.data('uiContextualappadsUiAppsdialog').dialog.find('.ep-appdialog-slider-wrapper .ep-appdialog-slider-teaser-link');
                appImgs =  $div.data('uiContextualappadsUiAppsdialog').dialog.find('.ep-appdialog-slider-wrapper img');

                mockApps.forEach( function(app, index) {
                    QUnit.equal(
                        appLinks.eq(index).attr('title'),
                        app.name,
                        'Dialog contains the following app name ' + app.name
                    );
                    QUnit.ok(
                        RegExp(app.logoUrl.replace(/\./g, "\\.")).test(appImgs.eq(index).attr('src')),
                        'Dialog contains the following app image ' + app.logoUrl
                    );
                })

                QUnit.equal(
                    $div.data('uiContextualappadsUiAppsdialog').dialog.find('.ep-appdialog-slider-wrapper .ep-appdialog-slider-teaser').length,
                    $div.data('uiContextualappadsUiAppsdialog').apps.length,
                    'Dialog contains ' + $div.data('uiContextualappadsUiAppsdialog').apps.length + ' App(s)'
                );

                // Close dialog
                $div.contextualappadsUiAppsdialog('close');

                setTimeout(function() {
                    QUnit.ok(
                        !$div.data('uiContextualappadsUiAppsdialog').dialog.is(':visible') ,
                        'Dialog is closed'
                    );

                    // Open dialog
                    $div.contextualappadsUiAppsdialog('open');
                    $(window).trigger('resize.contextualappadsUiAppsdialog')

                    setTimeout(function() {
                        QUnit.ok(
                            $div.data('uiContextualappadsUiAppsdialog').dialog.is(':visible') ,
                            'Dialog is open'
                        );

                        // test if _getPosition was called again after resize event
                        QUnit.ok(
                            resizeSpy.calledThrice,
                            'Dialogs position was recalculated after window resize.'
                        );

                        // Destroy widget
                        $div.contextualappadsUiAppsdialog('destroy');

                        setTimeout(function() {
                            QUnit.ok(
                                $div.data('uiContextualappadsUiAppsdialog') === undefined,
                                'Widget destroyed'
                            );
                            QUnit.ok(
                                !$('.ui-dialog').length,
                                'Dialog element was removed from DOM'
                            );

                            // restore spy after tests
                            resizeSpy.restore();

                            // start async tests
                            QUnit.start();
                        }, 300);
                    }, 200);
                }, 200);
            }, 400);
        }, 500);
    });
});